# EGEGrouper - Software for grouping electrogastroenterography examinations.

# Copyright (C) 2017-2018 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tabulate import tabulate

from .base_views import View, StatsView

def print_table(data, headers=None, title=None):
    """ Print table. """
    t = []
    for row in data:
        t_row = []
        for record in row:
            t_row.append(str(record)[:40])
        t.append(t_row)
    if title is not None:
        print('=== {} ==='.format(title))
    if headers:
        print('\n' + tabulate(t, headers=headers, tablefmt="orgtbl") + '\n')
    else:
        print('\n' + tabulate(t, tablefmt="orgtbl") + '\n')        

class MessageTextView(View):
    """Text message view."""
    def show_data(self, text):
        """ Print message. """
        print(text)

class StorageTextView(View):
    """ View showing information about storage. """
    def show_data(self, data, headers=None, title=None):
        print_table(data, headers=headers, title=title)
        
class GroupTextView(View):
    """ View showing information about group. """
    def show_data(self, data, headers=None, title=None):
        print_table(data, headers=headers, title=title)

class ExamTextView(View):
    """Text view to show details of examination."""
    def show_data(self, exam):
        """Print information about examination.
        
        Parameters
        ----------
        exam: sme.Examination
            SME examination object.

        """
        s = '\nE: {} {} {} {}\n'.format(exam.name, exam.gender, exam.age, exam.diagnosis)
        for m in exam.ms:
            s += '    M: {}\n'.format(m.time)
        print(s)

class WhereExamTextView(View):
    """Text view to show groups in which selected examination placed."""
    def show_data(self, group_records, headers, placed_in):
        """Show all groups and indicate where the object is located.

        Parameters
        ----------
        group_records : list of tuple
            Attributes of groups.
        headers : tuple
            Names of group attributes.
        placed_in : list of bool
            Indicators. True if examination placed in appropriate group, False overwise.

        """
        rows = [('X' if p else '', gr[0], gr[1]) for p, gr in zip(placed_in, group_records)]
        headers_ext = ('', ) + headers
        print('\n' + tabulate(rows, headers=headers_ext, tablefmt="orgtbl") + '\n')

class StatsTextView(StatsView):
    def show_data(self, data, headers):
        print_table(data, headers, self.title)



# EGEGrouper - Software for grouping electrogastroenterography examinations.

# Copyright (C) 2017-2018 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" This module implements the model for doing some descriptive
statistics. """

import numpy as np
from collections import OrderedDict

class StatsModel:
    """ The model class providing some descriptive statistics of
    groups. """
    def __init__(self, data_provider=None):
        """ Initialization. """
        # TODO: doc details
        self.data_provider = data_provider
        self.age_bounds = []
    
    def number_by_meta(self, key, exams=None):
        """ Group examinations by the meta data key and return the
        number of examinations in each group. """
        if exams is None:
            exams = self.data_provider.exams(group_id)
        res = OrderedDict()
        for e in exams:
            value = e._meta[key]
            if value not in res:
                res[value] = 1
            else:
                res[value] += 1
        return res

    def set_age_groups(self, bounds):
        """ Set bounds of age groups. """
        self.age_bounds = bounds

    def age_stats(self, group_id=None, exams=None):
        """ Return the average age of patients in the group. """
        if exams is None:
            exams = self.data_provider.exams(group_id)

        def get_key(b):
            return '{}-{}'.format(b[0], b[1])
        
        ages = [e.age for e in exams]
        res = OrderedDict()
        for b in self.age_bounds:
            res[get_key(b)] = 0
        for age in ages:
            for b in self.age_bounds:
                if b[0] <= age <= b[1]:
                    res[get_key(b)] += 1
        return res

    def stats(self, key, group_id):
        """ Return statistics for group by gender, diagnosis or age group. """
        es = self.data_provider.exams(group_id, meta_only=True)

        res = OrderedDict()
        if key in ['gender', 'diagnosis']:
            res = self.number_by_meta(key, exams=es)
        elif key == 'age':
            self.set_age_groups([(0,19), (20,29), (30,39), (40,49),
                                 (50,59), (60,69), (70,79), (80,100)])
            res = self.age_stats(exams=es)
            
        return res



# EGEGrouper - Software for grouping electrogastroenterography examinations.

# Copyright (C) 2017-2018 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import tkinter as tk
from tkinter import ttk, messagebox

from .base_views import View, StatsView

from .tk_widgets import TableWidget

class MessageTkView(View):
    """Text message view."""
    def show_data(self, text):
        """ Show message box.

        Parameters
        ----------
        text : str
            Message text.

        """
        messagebox.showinfo("Message", text)

class StorageTkView(View):
    """ View for showing information about storage. """
    def __init__(self):
        self._widget = None

    def get_widget(self):
        return self._widget
    def set_widget(self, widget):
        headers = ["ID", "Name", "Description", "Num"]
        widths = [50, None, None, 50]
        anchors = [tk.CENTER, None, None, tk.CENTER]
        widget.set_columns(headers=headers, widths=widths, anchors=anchors)
        self._widget = widget
    widget = property(get_widget, set_widget, doc="Table widget")

    def show_data(self, data, headers):
        self.widget.update_data(data)

class GroupTkView(View):
    """ View for showing information about group. """
    def __init__(self):
        self._widget = None

    def get_widget(self):
        return self._widget
    def set_widget(self, widget):
        headers = ["ID", "Name", "Diagnosis", "Age", "Gender"]
        widths = [50, None, None, None, None]
        anchors = [None, None, None, tk.CENTER, tk.CENTER]
        widget.set_columns(headers=headers, widths=widths, anchors=anchors)
        self._widget = widget
    widget = property(get_widget, set_widget, doc="Table widget")

    def show_data(self, data, headers):
        self.widget.update_data(data)

class StatsTkView(StatsView):
    def __init__(self, parent):
        """ Initialization. """
        super().__init__()
        self.parent = parent

    def show_data(self, data, headers):
        """ Show data. 

        Parameters
        ----------

        data: list of lists
            Table content.
        headers: list of str
            Headers.
        """
        table = self._build_window(self.parent)
        table.update_data(data)

    def _build_window(self, parent):
        """ Build and show window with table inside. """
        master = tk.Toplevel(parent)
        if self.title is not None:
            master.title(self.title)
        else:
            master.title("Statistics")
        table = TableWidget(master) 
        table.set_columns(
            headers=['Category', 'Number'],
            widths=[None, None],
            anchors=[None, tk.CENTER],
        )
        table.pack()
        master.transient(parent)
        return table

class WhereExamTkView(View):
    """ View showing in which groups the exam is located. """
    def __init__(self):
        self._widget = None
 
    def get_widget(self):
        return self._widget
    def set_widget(self, widget):
        self._widget = widget
    widget = property(get_widget, set_widget, doc="Grouping table widget")
    
    def show_data(self, group_records, headers, placed_in):
        rows = [
            (gr[0], gr[1], 'X' if p else '')
            for p, gr in zip(placed_in, group_records)
        ]
        headers_ext = headers + ('', )
        self.widget.update_data(rows)



# EGEGrouper - Software for grouping electrogastroenterography examinations.

# Copyright (C) 2017-2018 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" Controller for connection of user, StatsModel and views. """

class StatsController:
    """ Controller. """
    
    def __init__(self):
        """ Initialization. """
        self.model = None
        self.view = None
        self.status_view = None
        self.table_view = None
        self.message_view = None

    def stats(self, key, group_id):
        """ Calculate and show number of examinations grouped by key in group with group_id. 

        Parameters
        ----------
        key: str
            Keyword, name of field.
        group_id: str
            Group ID.

        """
        self.status_view.show_data("Calculating stats...")
        data = self.model.stats(key, group_id)
        if len(data) == 0:
            self.message_view.show_data("Result is empty")
            return
        table_data = [[key, data[key]] for key in data]
        self.table_view.title = "Stats by {}".format(key)
        self.table_view.show_data(table_data, ["Category", "Number"])
    
    def gender_balance(self, group_id):
        """ Calculate and show the gender balance of group. 

        Parameters
        ----------
        group_id: str
            Group ID.

        """
        data = self.model.gender_balance(group_id)
        self.view.show_data(data)



# EGEGrouper - Software for grouping electrogastroenterography examinations.

# Copyright (C) 2017-2018 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from abc import ABC, abstractmethod
import os
import sqlite3
import numpy as np

from . import sme
from . import sme_json
from . import sme_sqlite3

class BaseImporter(ABC):
    """Base class for importers."""
    
    def __init__(self, controller):
        """Constructor.

        Set controller to work with.
        
        """
        self.controller = controller

    def do_work(self, source):
        """Try import data from source and ask controller to put data into storage.
        
        Parameters
        ----------
        source : str
            Name of data source.
        
        """
        abs_file_name = os.path.expanduser(source)
        es = self._get_exams(abs_file_name)
        self.controller.import_exams(es)

    @abstractmethod
    def _get_exams(self, source):
        """Return exams from data source or None.

        Parameters
        ----------
        source : str
            Name of data source.

        Returns
        -------
        list of sme.Examination
            List of examinations.

        """
        pass
    

class JsonFileImporter(BaseImporter):
    """Importer from JSON file."""
    def _get_exams(self, source):
        exam = sme_json.get_exam(source)
        return [exam, ]

class SmeSqliteImporter(BaseImporter):
    """Importer from SME sqlite3 database."""
    def _get_exams(self, source):
        abs_file_name = os.path.expanduser(source)
        conn = sqlite3.connect(abs_file_name)
        c = conn.cursor()
        c.execute("SELECT exam_id from examination;")
        exams = []
        for r in c.fetchall():
            exam_id = r[0]
            exam = sme_sqlite3.get_exam(conn, exam_id)
            exams.append(exam)
        conn.close()
        return exams

class GsSqliteImporter(BaseImporter):
    """Importer from Gastroscan sqlite3 database."""
    def _get_exams(self, source):
        abs_file_name = os.path.expanduser(source)
        conn = sqlite3.connect(abs_file_name)
        c = conn.cursor()
        exams = []
        c.execute("""SELECT id, name, diagnosis, age, sex, date FROM record;""")
        for record_row in c.fetchall():
            e = sme.Examination()
            record_id = record_row[0]
            e.name = record_row[1]
            e.diagnosis = record_row[2]
            e.age = record_row[3]
            e.gender = record_row[4]
            m = sme.Measurement()
            m.time = record_row[5]
            m.ss = []
            c.execute("""
            SELECT signal
            FROM waveform
            WHERE (record_id = ?) and (edited = 0);""", (record_id, ))
            for signal_row in c.fetchall():
                s = sme.Signal()
                s.dt = 0.5
                s.x = np.array(np.frombuffer(signal_row[0]))
                m.ss.append(s)
            e.ms = [m]
            exams.append(e)
        conn.close()
        # try to join examinations
        joined_exams = exams
        # TODO
        return joined_exams



# EGEGrouper - Software for grouping electrogastroenterography examinations.

# Copyright (C) 2017-2018 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sqlite3
import numpy as np
from collections import OrderedDict
import os

from .base_model import BaseModel
from . import sme
from . import sme_sqlite3
from . import sme_json

class Model(BaseModel):
    """Model implementation for SQLite3 SME database."""
    
    def __init__(self):
        """Constructor.
        
        Create fields and set initial state of model.
        
        """
        super().__init__()
        self.conn = None
        self.c = None
        self.set_state(storage_opened=False, file_name = None)

    def create_storage(self, file_name):
        """Create new database.

        Parameters
        ----------
        file_name : str
            File name.

        """
        if self.state()['storage_opened']:
            self.close_storage()
        abs_file_name = os.path.expanduser(file_name)
        if os.path.isfile(abs_file_name):
            os.remove(abs_file_name)
        self.conn = sqlite3.connect(abs_file_name)
        self.c = self.conn.cursor()
        self.c.executescript(create_sme_db_script())
        self.conn.commit()
        self.set_state(storage_opened=True, file_name = abs_file_name)

    def open_storage(self, file_name):
        """Open database.

        Parameters
        ----------
        file_name : str
            File name.

        """
        if self.state()['storage_opened']:
            self.close_storage()
        abs_file_name = os.path.expanduser(file_name)
        self.conn = sqlite3.connect(abs_file_name)
        self.c = self.conn.cursor()
        self.c.execute("pragma foreign_keys=on")
        self.set_state(storage_opened=True, file_name=abs_file_name)

    def close_storage(self):
        """Close current database."""
        if self.state()['storage_opened']:
            self.conn.close()
            self.set_state(storage_opened=False, file_name=None)

    def storage_exists(self, file_name):
        """Check if the file exists.

        Parameters
        ----------
        file_name : str
            Name of database file.

        Returns
        -------
        : bool
            True if exists, False otherwise.

        """
        abs_file_name = os.path.expanduser(file_name)
        return os.path.isfile(abs_file_name)

    @BaseModel.do_if_storage_opened

    @BaseModel.do_if_storage_opened
    def storage_info(self):
        """Return tabular information about database.

        Returns
        -------
        ext_data : list of tuple
            Table with information about database.
        ext_headers : tuple
            Headers.

        """
        self.c.execute("""
        SELECT * FROM egeg_group;
        """)
        data = self.c.fetchall()
        headers = tuple(map(lambda x: x[0], self.c.description))
        num_in_groups = []

        ext_data = []
        for d in data:
            self.c.execute("""
            SELECT COUNT(E.exam_id)
            FROM examination as E, group_element as GE
            WHERE GE.exam_id = E.exam_id AND GE.group_id = ? """, [d[0], ])
            n = self.c.fetchall()[0][0]
            ext_data.append(d + (n,))
        ext_headers = headers + ("number",)

        self.c.execute("""
        SELECT COUNT(exam_id)
        FROM examination
        WHERE exam_id NOT IN (SELECT exam_id FROM group_element) """)
        ungrouped_num = self.c.fetchall()[0][0]
        
        last_string = ['' for h in ext_headers]
        last_string[0] = '0'
        last_string[-2] = 'Ungrouped'
        last_string[-1] = ungrouped_num
        ext_data.append(tuple(last_string))
        
        return ext_data, ext_headers    

    @BaseModel.do_if_storage_opened
    def group_info(self, group_id):
        """Return short information about examinations in selected group.

        Parameters
        ----------
        group_id : str
            Group ID

        Returns
        -------
        data : list of tuple
            Examination descriptions.
        headers : tuple
            Headers.

        """
        if group_id == '0':
            self.c.execute("""
            SELECT exam_id, name, diagnosis, age, gender
            FROM examination
            WHERE exam_id NOT IN (SELECT exam_id FROM group_element) 
            """)
        else:
            self.c.execute("""
            SELECT COUNT(*) FROM egeg_group WHERE group_id = ?
            """, [group_id])
            if self.c.fetchall()[0][0] == 0:
                return None, None

            self.c.execute("""
            SELECT E.exam_id, E.name, E.diagnosis, E.age, E.gender
            FROM examination AS E, group_element AS GE
            WHERE GE.exam_id = E.exam_id AND GE.group_id = ?
            """, [group_id, ])
            
        data = self.c.fetchall()
        headers = tuple(map(lambda x: x[0], self.c.description))
        return data, headers

    @BaseModel.do_if_storage_opened
    def insert_exam(self, exam):
        """Insert examination into current database.

        Parameters
        ----------
        exam : sme.Examination
            Examination object

        """
        sme_sqlite3.put_exam(self.conn, exam)

    @BaseModel.do_if_storage_opened
    def delete_exam(self, exam_id):
        """Remove examination from database.

        Parameters
        ----------
        exam_id : str
            Examination ID.

        """
        self.c.execute("""
        DELETE FROM Examination
        WHERE exam_id = ?
        """,(exam_id, ))
        self.conn.commit()

    @BaseModel.do_if_storage_opened
    def insert_group(self, name, description):
        """Add new group of examinations.

        Parameters
        ----------
        name : str
            Name of new group.
        description : str
            Description for new group.

        """
        self.c.execute("""
        INSERT INTO egeg_group (name, description)
        VALUES (?, ?) """, [name, description, ])
        self.conn.commit()

    @BaseModel.do_if_storage_opened
    def delete_group(self, group_id):
        """Delete group of examinations from database.

        Parameters
        ----------
        group_id : str
            Group ID.

        """
        self.c.execute("""
        DELETE FROM egeg_group
        WHERE group_id = ? """, [group_id, ])
        self.conn.commit()

    @BaseModel.do_if_storage_opened
    def group_exam(self, exam_id, group_ids, placed_in):
        """Add and delete examination to and from groups.

        Parameters
        ----------
        exam_id : str
            Examination ID.
        group_ids : list of str
            Group IDs.
        placed_in : list of bool
            True for examinations to be placed in groups. Length of group_ids must be equal to length of placed_in.

        """
        for (group_id, p) in zip(group_ids, placed_in):
            if p:
                try:
                    self.c.execute("""
                    INSERT OR IGNORE INTO group_element
                    VALUES (?, ?) """, [exam_id, group_id, ])
                except sqlite3.IntegrityError:
                    pass
            else:
                try:
                    self.c.execute("""
                    DELETE FROM group_element
                    WHERE exam_id = ? AND group_id = ? """, [exam_id, group_id, ])
                except sqlite3.IntegrityError:
                    pass
        self.conn.commit()

    @BaseModel.do_if_storage_opened
    def where_exam(self, exam_id):
        """Return description of groups where examination in or not in.

        Parameters
        ----------
        exam_id : str
            Examination ID.

        Returns
        -------
        group_records : list of tuple
            All group records.
        headers : list of str
            Names of group attributes.
        placed_in : list of bool
            True if exam in group, False otherwise.
        
        """

        including_exam_groups_ids = [
            r[0] for r in self.c.execute("""
            SELECT G.group_id
            FROM egeg_group as G, group_element
            WHERE G.group_id = group_element.group_id AND group_element.exam_id = ? """, [exam_id])
        ]

        self.c.execute("SELECT * FROM egeg_group")
        group_records = self.c.fetchall()
        headers = tuple(map(lambda x: x[0], self.c.description))
        placed_in = [gr[0] in including_exam_groups_ids for gr in group_records]
        
        return group_records, headers, placed_in

    @BaseModel.do_if_storage_opened
    def group_record(self, group_id):
        """Return attribute names and values of selected group.

        Parameters
        ----------
        group_id : str
            Group ID.

        Returns
        -------
        : OrderedDict
            Attributes names and values for selected group.

        """
        attr = OrderedDict()
        self.c.execute("""
        SELECT name, description
        FROM egeg_group
        WHERE group_id = ?
        """, (group_id, ))
        attr['name'], attr['description'] = self.c.fetchone()
        return attr

    @BaseModel.do_if_storage_opened
    def update_group_record(self, group_id, attr):
        """Update group record in database.

        Parameters
        ----------
        group_id : str
            Group ID.
        attr : OrderedDict
            Attributes names and values.

        """
        self.c.execute("""
        UPDATE egeg_group
        SET name = ?, description = ?
        WHERE group_id = ?
        """, (attr['name'], attr['description'], group_id, ))
        self.conn.commit()

    @BaseModel.do_if_storage_opened
    def exam(self, exam_id, meta_only=False):
        """Return examination from database.

        Parameters
        ----------
        exam_id : str
            Examination ID.
        meta_only : bool
            Get only meta data if True.

        Returns
        -------
        sme.Examination
            Examination object.

        """
        return sme_sqlite3.get_exam(self.conn, exam_id, meta_only)

    def __exam_ids(self, group_id):
        """ Return examination ids in group. """
        if group_id == '*':
            self.c.execute("SELECT exam_id FROM examination")
        elif group_id == '0':
            self.c.execute("""
            SELECT exam_id FROM examination
            WHERE exam_id NOT IN (SELECT exam_id FROM group_element) 
            """)
        else:
            self.c.execute("""
            SELECT E.exam_id FROM examination AS E, group_element AS GE
            WHERE GE.exam_id = E.exam_id AND GE.group_id = ?
            """, [group_id, ])
        ans = self.c.fetchall()
        if len(ans) == 0:
            return []
        res = [row[0] for row in ans]
        return res

    def __exams_of_group(self, group_id, meta_only=False):
        """ Return examination in group. """
        exam_ids = self.__exam_ids(group_id)
        res = [self.exam(exam_id, meta_only) for exam_id in exam_ids]
        return res

    def __exams_of_groups(self, group_ids, meta_only=False):
        """ Return examinations in several groups. """
        ids_lists = [self.__exam_ids(g_id) for g_id in group_ids]

        exam_ids_set = set(ids_lists[0])
        for ids_list in ids_lists[1:]:
            exam_ids_set = exam_ids_set | set(ids_list)
        exam_ids = list(exam_ids_set)

        es = [self.exam(exam_id) for exam_id in exam_ids]
        return es
    
    @BaseModel.do_if_storage_opened
    def exams(self, group_id, meta_only=False):
        """ Return exams from selected group or groups. 

        Parameters
        ----------
        group_id : str or list of str
            Group ID. If list, the union of sets of examinations from
            groups returned.
        meta_only: bool
            If True, only meta data returned.

        Returns
        -------
        exams: list of sme.Examination
            Examinations list.

        """
        if type(group_id) == type("1"):
            es = self.__exams_of_group(group_id, meta_only)
            return es
        es = self.__exams_of_groups(group_id, meta_only)
        return es

def create_sme_db_script():
    query = """
    pragma foreign_keys=1;

    create table signal(
        signal_id integer,
        data blob,
        dt real,
        edited integer, 
        meas_id integer references measurement(meas_id) on delete cascade, 
        primary key(signal_id)
    );

    create table measurement(
        meas_id integer, 
        time text, 
        exam_id integer references examination(exam_id) on delete cascade, 
        primary key(meas_id)
    );

    create table examination(
        exam_id integer, 
        name text,
        diagnosis text, 
        age integer, 
        gender text, 
        primary key(exam_id)
    );

    create table egeg_group(
        group_id integer, 
        name text, 
        description text,
        primary key(group_id)		
    );

    create table group_element(
        exam_id integer references examination(exam_id) on delete cascade, 
        group_id integer references egeg_group(group_id) on delete cascade, 
        primary key(exam_id, group_id)
    );
    """
    return query



# EGEGrouper - Software for grouping electrogastroenterography examinations.

# Copyright (C) 2017-2018 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tk interface to egegrouper."""

import tkinter as tk
from tkinter import messagebox, filedialog

from . import controller
from . import sqlite3_model
from . import tk_views, text_views
from . import plot_views
from . import importers
from . import tk_widgets
from .stats_model import StatsModel
from .stats_controller import StatsController

from .glob import *

from collections import OrderedDict

model = sqlite3_model.Model()
controller = controller.Controller(model)

class MainWindow:
    """Main window. Shows groups and main menu."""
    
    def __init__(self):
        """ Initialization. """
        self.master = tk.Tk()
        self.master.title("EGEGrouper {}".format(VERSION))

        self._make_main_menu()
        self.storage_menu.entryconfig("Import", state=tk.DISABLED)
        self.storage_menu.entryconfig("Close", state=tk.DISABLED)
        self.main_menu.entryconfig("Group", state=tk.DISABLED)
        self.main_menu.entryconfig("Exam", state=tk.DISABLED)
        self.group_menu.entryconfig("Statistics", state=tk.DISABLED)

        self.storage_table = tk_widgets.TableWidget(self.master)
        self.storage_table.pack(side=tk.LEFT, fill=tk.BOTH, expand=True)
        self.storage_table.item_opened.connect(self.group_info)
        self.storage_table.item_selected.connect(self.group_selected)
        storage_view = tk_views.StorageTkView()
        storage_view.widget = self.storage_table
        controller.set_view_storage(storage_view)
        self.group_window = GroupWindow(self.master)
        group_view = tk_views.GroupTkView()
        group_view.widget = self.group_window.group_table
        self.group_window.group_table.item_opened.connect(self.plot_exam)
        self.group_window.group_table.item_selected.connect(self.exam_selected)
        controller.set_view_group(group_view)
        controller.set_view_exam_plot(plot_views.ExamPlotView())

        stats_model = StatsModel()
        stats_model.data_provider = model
        self.stats_controller = StatsController()
        self.stats_controller.model = stats_model
        self.stats_controller.message_view = tk_views.MessageTkView()
        self.stats_controller.status_view = text_views.MessageTextView()
        self.stats_controller.table_view = tk_views.StatsTkView(self.master)

    def _make_main_menu(self):
        self.main_menu = tk.Menu(self.master)
        self.storage_menu = tk.Menu(self.main_menu, tearoff=0)
        self.storage_menu.add_command(label="Open", command=self.open_storage)
        self.storage_menu.add_command(label="Create", command=self.create_storage)
        self.storage_menu.add_command(label="Close", command=self.close_storage)
        self.add_data_submenu = tk.Menu(self.storage_menu, tearoff=0)
        self.add_data_submenu.add_command(label="SME sqlite3 DB", command=self.import_sme)
        self.add_data_submenu.add_command(label="Exam from JSON", command=self.import_json)
        #self.add_data_submenu.add_command(label="TODO: Add Gastroscan sqlite3 DB", command=)
        self.storage_menu.add_cascade(label="Import", menu=self.add_data_submenu)
        self.storage_menu.add_command(label="Exit", command=self.close_db_and_exit)
        self.main_menu.add_cascade(label="Storage", menu=self.storage_menu)
        self.group_menu = tk.Menu(self.main_menu, tearoff=0)
        self.group_menu.add_command(label="Add", command=self.add_group)
        self.group_menu.add_command(label="Edit", command=self.edit_group)
        self.group_menu.add_command(label="Delete", command=self.delete_group)
        self.stats_menu = tk.Menu(self.group_menu, tearoff=0)
        self.stats_menu.add_command(label="Gender", command=self.stats_gender)
        self.stats_menu.add_command(label="Diagnosis", command=self.stats_diagnosis)
        self.stats_menu.add_command(label="Age", command=self.stats_age)
        self.group_menu.add_cascade(label="Statistics", menu=self.stats_menu)
        self.main_menu.add_cascade(label="Group", menu=self.group_menu)
        self.exam_menu = tk.Menu(self.main_menu, tearoff=0)
        self.exam_menu.add_command(label="Plot", command=self.plot_exam)
        self.exam_menu.add_command(label="Grouping", command=self.grouping)
        self.exam_menu.add_command(label="Delete forever", command=self.delete_exam)
        #self.exam_menu.add_command(label="TODO?:Merge with", command=None)
        self.exam_menu.add_command(label="Export to JSON", command=self.export_json)
        self.main_menu.add_cascade(label="Exam", menu=self.exam_menu)
        self.help_menu = tk.Menu(self.main_menu, tearoff=0)
        self.help_menu.add_command(label="About", command=self.about)
        self.main_menu.add_cascade(label="Help", menu=self.help_menu)
        self.master.config(menu=self.main_menu)

    def open_storage(self):
        """Open storage and show groups in it."""
        file_name = filedialog.askopenfilename(
            title='Open storage',
            filetypes = [('sme db files', '.sme.sqlite'), ('all files', '.*')],
            parent = self.master,
        )
        if not file_name:
            return
        controller.open_storage(file_name)
        self.group_window.group_table.clear()
        # menu
        self.storage_menu.entryconfig("Import", state=tk.NORMAL)
        self.storage_menu.entryconfig("Close", state=tk.NORMAL)
        self.main_menu.entryconfig("Group", state=tk.NORMAL)
        self.group_menu.entryconfig("Edit", state=tk.DISABLED)
        self.group_menu.entryconfig("Delete", state=tk.DISABLED)
        self.group_menu.entryconfig("Statistics", state=tk.DISABLED)
        self.main_menu.entryconfig("Exam", state=tk.DISABLED)
        
    def create_storage(self):
        """Create storage."""
        file_name = filedialog.asksaveasfilename(
            title='Open storage',
            defaultextension = '.sme.sqlite',
            filetypes = [('sme db files', '.sme.sqlite'), ('all files', '.*')],
            parent = self.master,
        )
        if not file_name:
            return
        controller.create_storage(file_name)
        self.group_window.group_table.clear()
        # menu
        self.storage_menu.entryconfig("Import", state=tk.NORMAL)
        self.storage_menu.entryconfig("Close", state=tk.NORMAL)
        self.main_menu.entryconfig("Group", state=tk.NORMAL)
        self.group_menu.entryconfig("Edit", state=tk.DISABLED)
        self.group_menu.entryconfig("Delete", state=tk.DISABLED)
        self.group_menu.entryconfig("Statistics", state=tk.DISABLED)
        self.main_menu.entryconfig("Exam", state=tk.DISABLED)

    def close_storage(self):
        """Close storage and clear widgets."""
        controller.close_storage()
        self.group_window.group_table.clear()
        self.storage_table.clear()
        # menu
        self.storage_menu.entryconfig("Import", state=tk.DISABLED)
        self.storage_menu.entryconfig("Close", state=tk.DISABLED)
        self.main_menu.entryconfig("Group", state=tk.DISABLED)
        self.main_menu.entryconfig("Exam", state=tk.DISABLED)

    def import_sme(self):
        "Add records from sqlite3 data base in SME format."
        file_name = filedialog.askopenfilename(
            title='Open storage',
            filetypes = [('sme db files', '.sme.sqlite'), ('all files', '.*')],
            parent = self.master,
        )
        if not file_name:
            return
        importers.SmeSqliteImporter(controller).do_work(file_name)
        controller.storage_info()

    def import_json(self):
        """Add examination from JSON file."""
        file_name = filedialog.askopenfilename(
            parent = self.master,
            title = 'Add exam from JSON',
            filetypes = [('JSON files', '.json'), ('all files', '.*')],
        )
        if not file_name:
            return
        importers.JsonFileImporter(controller).do_work(file_name)
        controller.storage_info()

    def group_selected(self, *args):
        """Group selected slot. Enable some menu items."""
        # menu
        self.group_menu.entryconfig("Edit", state=tk.NORMAL)
        self.group_menu.entryconfig("Delete", state=tk.NORMAL)
        self.group_menu.entryconfig("Statistics", state=tk.NORMAL)

    def exam_selected(self, *args):
        """Exam selected slot. Enable some menu items."""
        # menu
        self.main_menu.entryconfig("Exam", state=tk.NORMAL)

    def group_info(self, *args):
        """Get and show information about examination in selected group."""
        if self.group_window.master.state() == "withdrawn":
            self.group_window.master.deiconify()
        group_id = self.storage_table.selected_item_text()
        if group_id:
            controller.group_info(group_id)
            self.storage_table.last_group_id = group_id
        # menu
        self.main_menu.entryconfig("Exam", state=tk.DISABLED)

    def grouping(self):
        """Open grouping dialog stub."""
        exam_id = self.group_window.group_table.selected_item_text()
        if not exam_id:
            return

        self.storage_table.remember_selection()
        self.group_window.group_table.remember_selection()
        grouping_dialog = GroupingDialog(self.master, exam_id)
        grouping_dialog.master.transient(self.master)
        grouping_dialog.master.grab_set()
        grouping_dialog.master.wait_window(grouping_dialog.master)
        controller.storage_info()
        controller.group_info(self.storage_table.last_group_id)
        self.storage_table.restore_selection()
        self.group_window.group_table.restore_selection()
        
    def delete_exam(self):
        """Delete exam from storage."""
        exam_id = self.group_window.group_table.selected_item_text()
        if not exam_id:
            return
        if messagebox.askquestion("Delete examination", "Are You shure?", icon='warning') == 'no':
            return
        controller.delete_exam(exam_id)
        self.storage_table.remember_selection()
        self.group_window.group_table.remember_selection()
        controller.storage_info()
        controller.group_info(self.storage_table.last_group_id)
        self.storage_table.restore_selection()
        self.group_window.group_table.restore_selection()

    def export_json(self):
        """Export selected examination to JSON file."""
        exam_id = self.group_window.group_table.selected_item_text()
        if not exam_id:
            return
        file_name = filedialog.asksaveasfilename(
            parent = self.master,
            title = 'Export exam to JSON',
            defaultextension = '.json',
            filetypes = [('JSON files', '.json'), ('all files', '.*')],
        )
        if not file_name:
            return
        controller.export_exam_to_json_file(exam_id, file_name)
        
    def add_group(self):
        """Add new group."""
        group_record_dialog = GroupRecordDialog(
            self.master,
            group_record = OrderedDict([('name',''), ('description','')])
        )
        group_record_dialog.master.transient(self.master)
        group_record_dialog.master.grab_set()
        group_record_dialog.master.wait_window(group_record_dialog.master)

    def edit_group(self):
        """Edit group."""
        group_id = self.storage_table.selected_item_text()
        if not group_id:
            return
        if group_id=='0':
            return
        self.storage_table.remember_selection()
        data = controller.group_record(group_id)
        group_record_dialog = GroupRecordDialog(self.master, data, group_id)
        group_record_dialog.master.transient(self.master)
        group_record_dialog.master.grab_set()
        group_record_dialog.master.wait_window(group_record_dialog.master)
        self.storage_table.restore_selection()

    def delete_group(self):
        """Delete selected group."""
        group_id = self.storage_table.selected_item_text()
        if not group_id:
            return
        if group_id=='0':
            return
        if messagebox.askquestion("Delete", "Are You shure?", icon='warning') == 'no':
            return
        self.storage_table.remember_selection()
        controller.delete_group(group_id)
        controller.storage_info()
        self.storage_table.restore_selection()

    def plot_exam(self, *args):
        """Plot examination in separate matplotlib window."""
        exam_id = self.group_window.group_table.selected_item_text()
        if exam_id:
            controller.plot_exam(exam_id)
                
    def close_db_and_exit(self):
        """Close data base and exit."""
        controller.close_storage()
        self.master.quit()

    def about(self):
        """Show info about program."""
        about_window = AboutWindow(self.master)
        about_window.master.transient(self.master)
        about_window.master.grab_set()
        about_window.master.wait_window(about_window.master)

    def stats_gender(self):
        """ Calculate and show statistics by gender. """
        group_id = self.storage_table.selected_item_text()
        self.stats_controller.stats('gender', group_id)
        
    def stats_diagnosis(self):
        """ Calculate and show statistics by diagnosis. """
        group_id = self.storage_table.selected_item_text()
        self.stats_controller.stats('diagnosis', group_id)
        
    def stats_age(self):
        """ Calculate and show statistics by age. """
        group_id = self.storage_table.selected_item_text()
        self.stats_controller.stats('age', group_id)

class GroupWindow:
    """Window for show and select examinations."""
    
    def __init__(self, parent):
        """Constructor.

        Create window.

        Parameters
        ----------
        parent
            Master for window.

        """
        self.master = tk.Toplevel(parent)
        self.master.title("Examinations")
        self.master.protocol('WM_DELETE_WINDOW', self.on_destroy)
        self.group_table = tk_widgets.TableWidget(self.master)
        self.group_table.pack(side=tk.LEFT, fill=tk.BOTH, expand=True)

    def on_destroy(self):
        """Do not destroy, but withdraw."""
        self.master.withdraw()

class GroupingDialog:
    """Dialog for grouping examinations."""
    
    def __init__(self, parent, exam_id):
        """Constructor.
        
        Parameters
        ----------
        parent
            Master.
        exam_id : str
            Examination ID.

        """
        self.exam_id = exam_id
        self.master = tk.Toplevel(parent)
        self.master.title("Grouping")
        self.grouping_widget = tk_widgets.GroupingTable(self.master)
        self.save_button = tk.Button(self.master, text="Save",
                                     width=15, command=self.on_save_button)
        self.cancel_button = tk.Button(self.master, text="Cancel",
                                       width=15, command=self.master.destroy)
        self.grouping_widget.pack()
        self.cancel_button.pack(side=tk.RIGHT)
        self.save_button.pack(side=tk.RIGHT)
        where_exam_view = tk_views.WhereExamTkView()
        where_exam_view.widget = self.grouping_widget
        controller.set_view_where_exam(where_exam_view)
        controller.where_exam(exam_id)

    def on_save_button(self):
        """Save button handler."""
        group_ids, placed_in = self.grouping_widget.checked_group_ids()
        controller.group_exam(self.exam_id, group_ids, placed_in)
        self.master.destroy()

class GroupRecordDialog:
    """Dialog for edit group record."""
    
    def __init__(self, parent, group_record, group_id = None):
        """Constructor.
        
        Parameters
        ----------
        parent
            Master.
        group_record : OrderedDict
            Attributes and values of group.
        group_is : str
            Group ID.

        """
        self.group_id = group_id
        self.group_record = group_record
        self.master = tk.Toplevel(parent)
        self.master.title("Edit group")
        self.labels = []
        self.entries = []
        for key in group_record:
            label = tk.Label(self.master, width = 10, text=key)
            label.pack(side=tk.TOP)
            entry = tk.Entry(self.master, width = 30)
            entry.delete(0, tk.END)
            if group_record[key]:
                entry.insert(0, group_record[key])
            else:
                entry.insert(0, '')
            entry.pack(side=tk.TOP)
            self.labels.append(label)
            self.entries.append(entry)
        self.save_button = tk.Button(self.master, text="Save",
                                     width=15, command=self.on_save_button)
        self.cancel_button = tk.Button(self.master, text="Cancel",
                                       width=15, command=self.master.destroy)
        self.cancel_button.pack(side=tk.RIGHT)
        self.save_button.pack(side=tk.RIGHT)

    def on_save_button(self):
        """Save button handler."""
        for (key, e) in zip(self.group_record, self.entries):
            self.group_record[key] = e.get()
        if not self.group_id:
            controller.insert_group(self.group_record['name'], self.group_record['description'])
        else:
            controller.update_group_record(self.group_id, self.group_record)
        controller.storage_info()
        self.master.destroy()

class AboutWindow:
    """Window with short info about the program."""
    def __init__(self, parent):
        """Constructor.

        Create window with info about the program.

        """
        self.master = tk.Toplevel(parent)
        self.master.title("About EGEGrouper")
        label = tk.Label(self.master, text="""
        EGEGrouper Copyright (C) 2017-2018 Aleksandr Popov

        This program comes with ABSOLUTELY NO WARRANTY.
        This is free software, and you are welcome to redistribute it
        under certain conditions.
        """)
        label.pack(side=tk.TOP)
        self.close_button = tk.Button(self.master, text="Close", width=15,
                                      command=self.master.destroy)
        self.close_button.pack(side=tk.TOP)

def main():
    """Entry point."""
    controller.set_view_message(tk_views.MessageTkView())
    
    main_window = MainWindow()
    main_window.master.mainloop()



# EGEGrouper - Software for grouping electrogastroenterography examinations.

# Copyright (C) 2017-2018 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Mapping examinations to and from sqlite3 SME database."""

import sqlite3
from . import sme
import numpy as np

def get_exam(conn, exam_id, only_meta=False):
    """Get examination object from database.

    Parameters
    ----------
    conn : sqlite3.Connection
        Opened connection to sqlite3 database.
    exam_id : str
        Examination ID.
    only_meta : bool
        Get only meta data of examination if True.
    
    Return
    ------
    : sme.Examination
        Examination instance.

    """
    # TODO: rename only_meta to meta_only
    e = sme.Examination()
    cursor = conn.cursor()
    cursor.execute("""
    SELECT E.name, E.diagnosis, E.age, E.gender FROM examination AS E
    WHERE exam_id = ? """, [exam_id, ])
    result = cursor.fetchone()
    if not result:
        return None
    e.name, e.diagnosis, e.age, e.gender = result
    
    if only_meta:
        return e
    
    e.ms = []
    cursor.execute("""
    SELECT M.meas_id, M.time FROM measurement AS M
    WHERE exam_id = ?
    ORDER BY meas_id """, [exam_id, ])
    for m_sql in cursor.fetchall():
        m = sme.Measurement()
        m_id, m.time = m_sql
        m.ss = []
        cursor.execute("""
        SELECT S.dt, S.data FROM signal AS S
        WHERE meas_id = ? """, [m_id, ])
        for s_sql in cursor.fetchall():
            s = sme.Signal()
            s.dt = s_sql[0]
            s.x = np.array(np.frombuffer(s_sql[1]))
            m.ss.append(s)
        e.ms.append(m)
    return e

def put_exam(conn, exam):
    """Put examination to database.

    Parameters
    ----------
    conn : sqlite3.Connection
        Opened connection with database.
    exam : sme.Examination
        Examination instance.
    
    """
    cursor = conn.cursor()
    cursor.execute("""
    SELECT max(exam_id)
    FROM examination """)
    exam_id = cursor.fetchone()[0]
    if not exam_id:
        exam_id = 0

    cursor.execute("""
    SELECT max(meas_id)
    FROM measurement """)
    meas_id = cursor.fetchone()[0]
    if not meas_id:
        meas_id = 0

    cursor.execute("""
    INSERT INTO examination (name, diagnosis, age, gender)
    VALUES (?,?,?,?) """, (exam.name, exam.diagnosis, exam.age, exam.gender) )
    exam_id += 1

    for m in exam.ms:
        cursor.execute("""
        INSERT INTO measurement (time, exam_id)
        VALUES (?,?) """, (m.time, exam_id) )
        meas_id += 1

        for s in m.ss:
            cursor.execute("""
            INSERT INTO signal (data, dt, meas_id)
            VALUES (?,?,?) """, (s.x.tobytes(), s.dt, meas_id) )

    conn.commit()



# EGEGrouper - Software for grouping electrogastroenterography examinations.

# Copyright (C) 2017-2018 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" Simple implementation of SME model. """

class Signal:
    """Signal."""

    def __init__(self):
        self._meta = {}
        self.x = []
    
    def set_dt(self, dt):
        self._meta['dt'] = dt
    def get_dt(self):
        return self._meta['dt']
    dt = property(get_dt, set_dt)    
    
class Measurement:
    """ Measurement which contains one or more signals. """
    def __init__(self):
        self._meta = {}
        self.ss = []

    def set_time(self, time):
        self._meta['time'] = time
    def get_time(self):
        return self._meta['time']
    time = property(get_time, set_time)    
    
class Examination:
    """ Examination, which contains one or more measurements. """
    def __init__(self):
        self._meta = {}
        self.ms = []

    def set_age(self, age):
        self._meta['age'] = age
    def get_age(self):
        return self._meta['age']
    age = property(get_age, set_age)

    def set_name(self, name):
        self._meta['name'] = name
    def get_name(self):
        return self._meta['name']
    name = property(get_name, set_name)
    
    def set_gender(self, gender):
        self._meta['gender'] = gender
    def get_gender(self):
        return self._meta['gender']
    gender = property(get_gender, set_gender)
    
    def set_diagnosis(self, diagnosis):
        self._meta['diagnosis'] = diagnosis
    def get_diagnosis(self):
        return self._meta['diagnosis']
    diagnosis = property(get_diagnosis, set_diagnosis)
    
def merge_exams(e1, e2):
    """Merge exams."""
    e = Examination()
    e = e1
    for m in e2.ms:
        e1.ms.append(m)
    return e



# EGEGrouper - Software for grouping electrogastroenterography examinations.

# Copyright (C) 2017-2018 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from setuptools import setup
import os

from egegrouper.glob import *

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

name="EGEGrouper"
description="Tool for grouping EGEG examinations"
author="Aleksandr Popov"
author_email="aleneus@gmail.com"
license = "GPLv3"
keywords = "electrophysiology electrogastrography electrogastroenterography biosignal storage dataset"
url = "https://bitbucket.org/aleneus/egegrouper"
packages=['egegrouper']

install_requires = [
    'numpy',
    'matplotlib',
    'tabulate',
    'nose',
    'pyreadline;platform_system=="Windows"',
]

entry_points={
    'console_scripts': [
        'igrouper = egegrouper.igrouper:main'
    ],
    'gui_scripts': [
        'tkgrouper = egegrouper.tkgrouper:main'
    ]
}

classifiers=[
    "Development Status :: 4 - Beta",
    "Environment :: Console",
    "Environment :: X11 Applications",
    "Intended Audience :: Science/Research",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    "Operating System :: POSIX :: Linux",
    "Operating System :: Microsoft :: Windows",
    "Programming Language :: Python :: 3",
    "Topic :: Scientific/Engineering :: Bio-Informatics",
]

def linux_setup():
    setup(
        name=name,
        version=VERSION,
        description=description,
        author=author,
        author_email=author_email,
        license = license,
        keywords = keywords,
        url = url,
        long_description=read('README'),
        packages=packages,
        install_requires = install_requires,
        entry_points=entry_points,
        classifiers=classifiers,
    )

def windows_build():
    pass

linux_setup()
#from cx_Freeze import setup, Executable
#windows_build()



# EGEGrouper - Software for grouping electrogastroenterography examinations.

# Copyright (C) 2017-2018 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from . import sme_json
import os

class BaseExporter:
    """Base class for exporters."""

    def export_exam(self, exam, dest_name):
        """ Export examination to destination with dest_name.

        Parameters
        ----------
        exam : sme.Examination
            Examination instance.
        dest_name : str
            Destination name.

        """
        pass

class JsonFileExporter(BaseExporter):
    """Exporter to JSON file."""

    def export_exam(self, exam, dest_name):
        """ Export examination to JSON file with dest_name.

        Parameters
        ----------
        exam : sme.Examination
            Examination instance.
        dest_name : str
            JSON file name.

        """
        abs_file_name = os.path.expanduser(dest_name)
        sme_json.put_exam(exam, abs_file_name)
